﻿using System;
using System.Collections.Generic;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Security.Cryptography.Xml;
using System.Text;
using System.Threading.Tasks;

namespace KPGBase.Excercises
{
    internal class Excercise04 : BasicExcercise
    {

        public Excercise04(Bitmap canvas) : base(canvas) { }

        /// <summary>
        /// Fill the plane with 12-gonal tiles.
        /// </summary>
        public void SimpleTiling()
        {
            ClearImage();
            Matrix startMatrix = g.Transform.Clone();

            // HERE INSERT YOUR CODE

            Brush trigB = new SolidBrush(Color.GreenYellow);
            g.FillRectangle(trigB, new Rectangle(0, 0, width, height));

            // parameters
            int n = 12;
            int radius = 120; // radius of n-gon, distance from center to one point
            double alpha = (360.0 / n) / 2; // angle between two points and center
            int v = (int)(Math.Cos(alpha * Math.PI / 180) * radius / 2.0); // distance from center to one side

            // reference n-gon to compute needed shift
            Point[] pts = CreateNgon(0, 0, n, radius / 2);
            int shift = (int)Math.Abs(pts[9].Y - pts[10].Y); // distance of points at the top of the n-gon, needed for shift

            // used brushes
            Brush hexB = new SolidBrush(Color.DarkRed);
            Pen line = new Pen(Color.Black, 2);

            int hexLocY = 0;
            int row = 1;

            // save trasform matrix
            Matrix mat = g.Transform.Clone();

            // rows
            while (hexLocY < (height + v))
            {
                // every other row has to be shifted
                int hexLocX = v;
                if (row % 2 == 0)
                    hexLocX = 0;

                // translate to the starting point
                g.Transform = mat;
                g.TranslateTransform(hexLocX, hexLocY);

                // columns
                while (hexLocX < (width + v))
                {
                    g.FillPolygon(hexB, pts);
                    g.DrawPolygon(line, pts);
                    hexLocX += 2 * v;
                    
                    // translate by width of ngon
                    g.TranslateTransform(2*v, 0);
                }
                hexLocY += 2 * v - shift;
                row++;
            }

            // set transform back to original
            g.Transform = startMatrix;
        }

        /// <summary>
        /// Creates an n-gon at location [x, y]
        /// </summary>
        /// <param name="x"> X coordinate </param>
        /// <param name="y"> Y coordinate </param>
        /// <param name="n"> Number of points </param>
        /// <param name="r"> Radius </param>
        /// <returns> Points of the generated n-gon </returns>
        Point[] CreateNgon(int x, int y, int n, int r)
        {
            Point[] points = new Point[n];
            double step = 360 / n * Math.PI / 180f;
            for (int i = 0; i < n; i++)
            {
                points[i] = new Point((int)(x + r * (float)Math.Cos(i * 360 / n * Math.PI / 180f + step / 2)),
                                       (int)(y + r * (float)Math.Sin(i * 360 / n * Math.PI / 180f + step / 2)));
            }
            return points;
        }

        public void TriangleTiling()
        {
            ClearImage();
            Matrix startMatrix = g.Transform.Clone();

            throw new NotImplementedException("Triangle tiles not implemented");

            g.Transform = startMatrix;
        }

        public void SpiralTiling()
        {
            ClearImage();
            Matrix startMatrix = g.Transform.Clone();

            throw new NotImplementedException("Spiral tiles not implemented");

            g.Transform = startMatrix;
        }

    }
}
